// File:   t_rl_DielectricPOD_rdb.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_ray
//
// rl_ray is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_ray is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

// t_rl_dielectricpod_rdb.cc

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <iostream>
#include <string>

#include "rl_raysuplib/rl_DielectricPOD_rdb.h"

#include <mathconst/mathconst.h>
#include <tracefctxx/TraceFct.h>

using namespace std;

static
void rl_DielectricPOD_rdb_test01(char const* s1,
                                 char const* outfile)    /* test ctor, dtor */
{
  rl_DielectricPOD_rdb diel( s1 );

  FILE* out = fopen(outfile, "w");
  if (! out )
  {
    fprintf(stderr, "Failed to open output file %s\n", outfile);
    exit(EXIT_FAILURE);
  } 

  diel.cprint_on(out,
                 static_cast<char const*>(""),
                 static_cast<char const*>("") );
}

int
main( int argc, char* argv[] )
{
  string inputdir;
  char*  inputdir_env = NULL; // must keep around as putenv uses its memory

  if ( argc == 2 )
  {
    string temp = string("INPUTDIR=") + argv[1];
    inputdir_env = (char*) malloc( temp.length() + 1 );
    strcpy( inputdir_env, temp.c_str() );
    putenv( inputdir_env );

    // always use interpolation for tests.
    inputdir = "${INPUTDIR}/";
  }

  else if ( NULL != getenv( "INPUTDIR" ) )
  {
    inputdir = "${INPUTDIR}/";
  }

  else
  {
    cerr << "Usage: " << argv[0] << " inputdir\n";
    return EXIT_FAILURE;
  }

  string src = inputdir + "Ir.dat";

  rl_DielectricPOD_rdb_test01(src.c_str(), "t_rl_DielectricPOD_rdb.log");

  if ( inputdir_env )
    free( inputdir_env );
  return EXIT_SUCCESS;
}
