// File:   t_rl_multilayer_rdb.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_ray
//
// rl_ray is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_ray is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <string>

#include <cstdlib>
#include <cstring>
#include <iostream>


#include "rl_raysuplib/rl_Multilayer_rdb.h"
#include "rl_raylib/rl_ReflectionCoefPOD.h"

#include <rl_raysuplib/rl_DielectricPOD_rdb.h>

#include <mathconst/mathconst.h>

using namespace std;

static
void rl_Multilayer_rdb_test01(FILE* out)    /* test ctor, dtor */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect.rdb",  rl_Traits::ELinLin );

  ml.cdump_on( out, 0, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
  ml.cdump_on( out, 1, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
  ml.cdump_on( out, 2, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
}

static
int rl_Multilayer_rdb_test02(FILE* out)    
   /* test multilayer_reflect_coef() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect.rdb",  rl_Traits::ELinLin );

  rl_Traits::complex para;
  rl_Traits::complex perp;

  double energy = 7.0;
  double sg     = 0.015126174;

  rl_ReflectionCoefPOD rflcoef;
  int rc = ml.multilayer_reflect_coef( rflcoef, energy, sg );

  rflcoef.cprint_on(out);

  return !rc;
}

static
int rl_Multilayer_rdb_test03(FILE* out)    
   /* test multilayer_reflectivity() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect.rdb",  rl_Traits::ELinLin );

  double energy = 7.0;
  double sg     = 0.015126174;

  double rfl = 0;
  int rc = ml.multilayer_reflectivity( rfl, energy, sg );

  fprintf(out, "RFL %.15e\n", rfl);

  return !rc;
}

static
int rl_Multilayer_rdb_test04(char const* s1, FILE* out)  
           /* test multilayer_reflectivity() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect2.rdb",  rl_Traits::ELinLin );

  rl_DielectricPOD_rdb diel( s1 );

  rl_Traits::rl_DielectricPOD const* data = diel.const_data_ptr();

  double sg[] = { 51.2, 41.2, 36.4, 27.0 };

  ml.cdump_on( out, 0, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
  ml.cdump_on( out, 1, const_cast<char const*>(""), 
                       const_cast<char const*>("") );
  ml.cdump_on( out, 2, const_cast<char const*>(""), 
                       const_cast<char const*>("") );

  int n;
  double const deg2rad = 3.14159265358979323846 / 180.0;
  for ( n = 0 ; n < 4 ; ++n )
  {
    sg[n]  = sin( deg2rad * sg[n] / 60.0 );
  }
  int rc = 1;
  for ( n = 0 ; n < diel.num_elts() ; ++n )
  {
    double rfl1 = 0.0;
    double rfl2 = 0.0;
    double rfl3 = 0.0;
    double rfl4 = 0.0;
    rc    &= !(ml.multilayer_reflectivity(rfl1, data[n].energy_, sg[0]));
    rc    &= !(ml.multilayer_reflectivity(rfl2, data[n].energy_, sg[1]));
    int i = ml.multilayer_reflectivity(rfl3, data[n].energy_, sg[2]);
    i = ml.multilayer_reflectivity(rfl4, data[n].energy_, sg[3]);
    fprintf(out, "%d\t%d\t%15.9e\t%15.9e\t%15.9e\t%15.9e\t%15.9e\n", 
             rc, n, data[n].energy_, rfl1, rfl2, rfl3, rfl4 );
  }
  return rc;
}

static
int rl_Multilayer_rdb_test05(char const* s2, FILE* out) 
            /* test multilayer_reflectivity() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect2.rdb", rl_Traits::ELinLin );

  rl_DielectricPOD_rdb diel( s2 );

  double sg[] = { 51.2, 41.2, 36.4, 27.0 };

  int n;
  double const deg2rad = 3.14159265358979323846 / 180.0;
  for ( n = 0 ; n < 4 ; ++n )
  {
    sg[n]  = sin( deg2rad * sg[n] / 60.0 );
  }

  double const energy = 7.999996;
  // double const energy = 5.000006;

  // double fac1   = ml.multilayer_reflectivity( energy, sg[0] );

  int rc = 1;
  {
    double rfl;
    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[0] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[0], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[1] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[1], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[2] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[2], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[3] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[3], rfl);
  }
  return rc;
}

static
int rl_Multilayer_rdb_test06(char const* s2, FILE* out) 
                   /* test multilayer_reflectivity() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect3.rdb",  rl_Traits::ELinLin );

  rl_DielectricPOD_rdb diel( s2 );

  double sg[] = { 51.2, 41.2, 36.4, 27.0 };

  int n;
  double const deg2rad = 3.14159265358979323846 / 180.0;
  for ( n = 0 ; n < 4 ; ++n )
  {
    sg[n]  = sin( deg2rad * sg[n] / 60.0 );
  }

  double const energy = 5.000006;

  int rc = 1;
  {
    double rfl;
    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[0] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[0], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[1] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[1], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[2] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[2], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[3] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[3], rfl);
  }

  return rc;
}

static
int rl_Multilayer_rdb_test07(char const* s2, FILE* out) 
         /* test multilayer_reflectivity() */
{
  rl_Multilayer_rdb ml( "${INPUTDIR}/reflect3.rdb",  rl_Traits::ELinLin );

  rl_DielectricPOD_rdb diel( s2 );

  double sg[] = { 51.2, 41.2, 36.4, 27.0 };

  int n;
  double const deg2rad = 3.14159265358979323846 / 180.0;
  for ( n = 0 ; n < 4 ; ++n )
  {
    sg[n]  = sin( deg2rad * sg[n] / 60.0 );
  }

  // double const energy = 0.042378;
  // double const energy = 7.999996;
  double const energy = 5.000006;

  // double fac1   = ml.multilayer_reflectivity( energy, sg[0] );

  int rc = 1;
  {
    double rfl;
    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[0] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[0], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[1] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[1], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[2] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[2], rfl);

    rc &= !(ml.multilayer_reflectivity( rfl, energy, sg[3] ));
    fprintf(out, "%15.9e\t%15.9e\t%15.9e\n", energy, sg[3], rfl);
  }

  return rc;
}

int
main( int argc, char* argv[] )
{
  string inputdir;
  char*  inputdir_env = NULL; // must keep around as putenv uses its memory

  if ( argc == 2 )
  {
    string temp = string("INPUTDIR=") + argv[1];
    inputdir_env = (char*) malloc( temp.length() + 1 );
    strcpy( inputdir_env, temp.c_str() );
    putenv( inputdir_env );

    // always use interpolation for tests
    inputdir = "${INPUTDIR}/";
  }

  else if ( NULL != getenv( "INPUTDIR" ) )
  {
    inputdir = "${INPUTDIR}/";
  }

  else
  {
    cerr << "Usage: " << argv[0] << " inputdir\n";
    return EXIT_FAILURE;
  }

  string src1 = inputdir + "ir_data.rdb";
  string src2 = inputdir + "Ir_14.dat";

  FILE* out = fopen("ml_test_rdb.out", "w");

  int rc  = 1;
  rl_Multilayer_rdb_test01(out);
  rc &= rl_Multilayer_rdb_test02(out);
  rc &= rl_Multilayer_rdb_test03(out);
  rc &= rl_Multilayer_rdb_test04(src1.c_str(),out);
  rc &= rl_Multilayer_rdb_test05(src1.c_str(),out);
  rc &= rl_Multilayer_rdb_test06(src2.c_str(),out);
  rc &= rl_Multilayer_rdb_test07(src2.c_str(),out);

  fclose(out);

  if ( inputdir_env )
    free( inputdir_env );

  return !rc;
}

